
import { Mixin, mix } from "../../mixwith.js";
import { CSSConstructionMixin } from "../css_construction_mixin.js";

import { CSSProperty } from "./css_property/css_property.js";
import { CSSBoxProperty } from "./css_property/css_box_property.js";

/*
    A single CSS Expression.
*/
export class CSSExpression extends mix(Object).with(CSSConstructionMixin) {

  static get empty() {
    if ( ! this._empty ) {
      this._empty = new this();
    }
    return this._empty;
  }

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_css_constructor, ...css_property_objects  ) {
    super(parent_css_constructor);
    this._properties = {};
    this.initCSSProperties( ...css_property_objects );
  }

  initCSSProperties( ...css_property_objects ) {
    for ( let this_property_object of css_property_objects ) {
      this.setPropertyObject( this_property_object );
    }
  }

  /***************
  *  Properties  *
  ***************/

  set properties( css_properties_object) {
    this.clearCSS();
    this._properties = css_properties_object;
  }

  get properties() {
    return this._properties;
  }

  property( property_name ) {
    return this._properties[ property_name ];
  }

  setProperty( property_name, property_value ) {
    this.clearCSS();
    let new_property = new CSSProperty( this, property_name, property_value );
    this._properties[property_name] = new_property;
    return new_property;
  }

  setBoxProperty( property_name, property_value ) {
    this.clearCSS();
    let new_property = new CSSBoxProperty( this, property_name, property_value );
    this._properties[property_name] = new_property;
    return new_property;
  }

  setPropertyObject( css_property_object ) {
    this.clearCSS();
    this._properties[css_property_object.propertyName] = css_property_object;
  }

  /***********
  *  Output  *
  ***********/

  get empty() {
    return this.constructor.empty;
  }

  get css() {
    if ( ! this._css ) {
      this._css = '';
      let object_properties = Object.getOwnPropertyNames( this.properties );
      for ( let this_property_name of object_properties ) {
        let this_property = this.properties[this_property_name];
        this._css += "\t" + this_property.css + "\n";
      }
      this._css = "{" + (this._css.length ? "\n" + this._css : "") + "}";
    }
    return this._css;
  }

}

export default CSSExpression;
