
import {HostMixin} from "../../abstract/host_mixin.js";

import { Mixin, mix } from "../../mixwith.js";
import "../../object/id.js";

import { CSSRule } from '../../css_rule/css_rule.js';

/*
    Controls a stylesheet and enables or disables a rule.
*/
export class CSSRuleController extends mix(Object).with(HostMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( host, sheet_controller, css_rule_index ) {
    super( host )
    this._sheetController = sheet_controller;
    this._cssRule = new CSSRule( this );
    this._cssRuleIndex = css_rule_index;
  }

  /**************
  *  CSS State  *
  **************/

  enable() {
    this.enabled = true;
  }

  disable() {
    this.enabled = false;
  }

  toggle() {
    this.enabled = ! this.enabled;
  }

  set enabled( true_or_false ) {
    this._enabled = true_or_false;
    this.enabledDidChange();
  }

  get enabled() {
    return this._enabled;
  }

  set disabled( true_or_false ) {
    this.enabled = ! true_or_false;
  }

  get disabled() {
    return ! this._enabled;
  }

  /**************
  *  CSS Rules  *
  **************/

  set ruleIndex( css_rule_index ) {
    this._cssRuleIndex = css_rule_index;
  }

  get ruleIndex() {
    return this._cssRuleIndex;
  }

  set rule( css_rule ) {
    this._cssRule = css_rule;
  }

  get rule() {
    return this._cssRule;
  }

  get css() {
    return this.rule.css;
  }

  updateRuleInSheet() {
    this.sheetController.sheet.cssRules[this.ruleIndex].cssText = this.rule.css;
  }

  updateSelectorInSheet() {
    this.sheetController.sheet.cssRules[this.ruleIndex].selectorText = this.rule.selector.css;
  }

  /***********
  *  Events  *
  ***********/

  ruleDidChange() {
  }

  enabledDidChange() {
    if ( this.enabled ) {
      this.updateRuleInSheet();
    }
    else {
      this.ruleInSheet.expression = this.rule.expression.empty.css;
    }
  }

  clearCSS() {
    this.ruleDidChange();
  }

  /***************
  *  Properties  *
  ***************/

  set sheetController( sheet_controller ) {
    this._sheetController = sheet_controller;
  }

  get sheetController() {
    return this._sheetController;
  }

  get ruleInSheet() {
    return this._ruleInSheet;
  }

  set ruleInSheet( rule_in_sheet ) {
    return this._ruleInSheet = rule_in_sheet;
  }

  get ruleInSheet() {
    return this._ruleInSheet;
  }

  set selector( selector ) {
    this.rule.selector = selector;
  }

  get selector() {
    return this.rule.selector;
  }

  set expression( expression ) {
    this.rule.expression = expression;
  }

  get expression() {
    return this.rule.expression;
  }

}

export default CSSRuleController;
