
import { Mixin, mix } from "../../mixwith.js";
import {CSSSelectorConstructionMixin} from "./css_selector_construction_mixin.js";

import {CSSAttributeSelector} from "./selector/css_attribute_selector.js";
import {CSSContentSelector} from "./selector/css_content_selector.js";
import {CSSInputSelector} from "./selector/css_input_selector.js";
import {CSSRelationSelector} from "./selector/css_relation_selector.js";
import {CSSStateSelector} from "./selector/css_state_selector.js";
import {CSSTagSelector} from "./selector/css_tag_selector.js";
import {CSSTypeSelector} from "./selector/css_type_selector.js";

/*
    A single selector for all individual element properties but without element
    relation selectors.
*/
export class CSSElementSelector extends mix(Object).with(CSSSelectorConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_selector = undefined ) {
    super( parent_selector );
    this._selectors = {};
  }

  /********************
  *  Selector Output  *
  ********************/

  get css() {
    if ( ! this._css ) {

      this._css = '';

      if ( this._selectors.tagSelector ) {
        this._css += this._selectors.tagSelector.css;
      }

      if ( this._selectors.attributeSelectors ) {
        let attributes = Object.getOwnPropertyNames(this._selectors.attributeSelectors);
        for ( let this_attribute of attributes ) {
          let this_attribute_selector = this._selectors.attributeSelectors[this_attribute];
          this._css += this_attribute_selector.css;
        }
      }

      if ( this._selectors.relationSelector ) {
        this._css += this._selectors.relationSelector.css;
      }

      if ( this._selectors.typeSelector ) {
        this._css += this._selectors.typeSelector.css;
      }

      if ( this._selectors.stateSelector ) {
        this._css += this._selectors.stateSelector.css;
      }

      if ( this._selectors.inputSelector ) {
        this._css += this._selectors.inputSelector.css;
      }

      if ( this._selectors.contentSelector ) {
        this._css += this._selectors.contentSelector.css;
      }

      if ( this._selectors.notTagSelector ) {
        this._css += this._selectors.notTagSelector.css;
      }

      if ( this._selectors.notAttributeSelectors ) {
        let attributes = Object.getOwnPropertyNames(this._selectors.notAttributeSelectors);
        for ( let this_attribute of attributes ) {
          let this_attribute_selector = this._selectors.notAttributeSelectors[this_attribute];
          this._css += this_attribute_selector.css;
        }
      }

      if ( this._selectors.notInputSelector ) {
        this._css += this._selectors.notInputSelector.css;
      }

      if ( this._selectors.notStateSelector ) {
        this._css += this._selectors.notStateSelector.css;
      }

      if ( this._selectors.notTypeSelector ) {
        this._css += this._selectors.notTypeSelector.css;
      }

      if ( this._selectors.notRelationSelector ) {
        this._css += this._selectors.notRelationSelector.css;
      }

      if ( this._selectors.notContentSelector ) {
        this._css += this._selectors.notContentSelector.css;
      }

      this._css = super.css;

    }

    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  set tagSelector( tag_selector ) {
    this._selectors.tagSelector = tag_selector;
  }

  get tagSelector() {
    if ( ! this._selectors.tagSelector ) {
      this.tagSelector = new CSSTagSelector( this );
    }
    return this._selectors.tagSelector;
  }

  set notTagSelector( not_tag_selector ) {
    not_tag_selector.not = true;
    this._selectors.notTagSelector = not_tag_selector;
  }

  get notTagSelector() {
    if ( ! this._selectors.notTagSelector ) {
      this.notTagSelector = new CSSTagSelector( this );
    }
    return this._selectors.notTagSelector;
  }

  set attributeSelectors( attribute_selectors ) {
    this._selectors.attributeSelectors = [];
    this.addAttributeSelectorInstances( ...attribute_selectors );
  }

  addAttributeSelectorInstances( ...attribute_selectors ) {
    for ( let this_attribute_selector of attribute_selectors ) {
      this.addAttributeSelectorInstance( this_attribute_selector );
    }
  }

  addAttributeSelectorInstance( attribute_selector ) {
    this.attributeSelectors[attribute_selector.attributeName] = attribute_selector;
    return attribute_selector;
  }

  get attributeSelectors() {
    if ( ! this._selectors.attributeSelectors ) {
      this._selectors.attributeSelectors = {};
    }
    return this._selectors.attributeSelectors;
  }

  attributeSelector( attribute_name ) {
    return this._selectors.attributeSelectors ?
           this._selectors.attributeSelectors[attribute_name] : undefined;
  }

  ensureAttributeSelector( attribute_name ) {
    return this._selectors.attributeSelectors ?
           this._selectors.attributeSelectors[attribute_name] :
           this.addAttributeSelector( attribute_name );
  }

  addAttributeSelector( attribute_name ) {
    let attribute_selector = new CSSAttributeSelector( attribute_name, this );
    this.addAttributeSelectorInstance( attribute_selector );
    return attribute_selector;
  }

  set notAttributeSelectors( not_attribute_selectors ) {
    this._selectors.notAttributeSelectors = [];
    this.addNotAttributeSelectorInstances( ...attribute_selectors );
  }

  addNotAttributeSelectorInstances( ...attribute_selectors ) {
    for ( let this_attribute_selector of attribute_selectors ) {
      this.addNotAttributeSelectorInstance( this_attribute_selector );
    }
  }

  addNotAttributeSelectorInstance( attribute_selector ) {
    attribute_selector.not = true;
    this.notAttributeSelectors[attribute_selector.attributeName] = attribute_selector;
    return attribute_selector;
  }

  get notAttributeSelectors() {
    if ( ! this._selectors.notAttributeSelectors ) {
      this._selectors.notAttributeSelectors = {};
    }
    return this._selectors.notAttributeSelectors;
  }

  notAttributeSelector( attribute_name ) {
    return this._selectors.notAttributeSelectors ?
           this._selectors.notAttributeSelectors[attribute_name] : undefined;
  }

  addNotAttributeSelector( attribute_name ) {
    let attribute_selector = new CSSAttributeSelector( attribute_name, this );
    this.addNotAttributeSelectorInstance( attribute_selector );
    return attribute_selector;
  }

  ensureNotAttributeSelector( attribute_name ) {
    return this._selectors.notAttributeSelectors ?
           this._selectors.notAttributeSelectors[attribute_name] :
           this.addNotAttributeSelector( attribute_name );
  }

  set inputSelector( input_selector ) {
    this._selectors.inputSelector = input_selector;
  }

  get inputSelector() {
    if ( ! this._selectors.inputSelector ) {
      this.inputSelector = new CSSInputSelector( this );
    }
    return this._selectors.inputSelector;
  }

  set notInputSelector( not_input_selector ) {
    not_input_selector.not = true;
    this._selectors.notInputSelector = not_input_selector;
  }

  get notInputSelector() {
    if ( ! this._selectors.notInputSelector ) {
      this.notInputSelector = new CSSInputSelector( this );
    }
    return this._selectors.notInputSelector;
  }

  set stateSelector( state_selector ) {
    this._selectors.stateSelector = state_selector;
  }

  get stateSelector() {
    if ( ! this._selectors.stateSelector ) {
      this.stateSelector = new CSSStateSelector( this );
    }
    return this._selectors.stateSelector;
  }

  set notStateSelector( not_state_selector ) {
    not_state_selector.not = true;
    this._selectors.notStateSelector = not_state_selector;
  }

  get notStateSelector() {
    if ( ! this._selectors.notStateSelector ) {
      this.notStateSelector = new CSSStateSelector( this );
    }
    return this._selectors.notStateSelector;
  }

  set typeSelector( type_selector ) {
    this._selectors.typeSelector = type_selector;
  }

  get typeSelector() {
    if ( ! this._selectors.typeSelector ) {
      this.typeSelector = new CSSTypeSelector( this );
    }
    return this._selectors.typeSelector;
  }

  set notTypeSelector( not_type_selector ) {
    not_type_selector.not = true;
    this._selectors.notTypeSelector = not_type_selector;
  }

  get notTypeSelector() {
    if ( ! this._selectors.notTypeSelector ) {
      this.notTypeSelector = new CSSTypeSelector( this );
    }
    return this._selectors.notTypeSelector;
  }

  set relationSelector( relation_selector ) {
    this._selectors.relationSelector = relation_selector;
  }

  get relationSelector() {
    if ( ! this._selectors.relationSelector ) {
      this.relationSelector = new CSSRelationSelector( this );
    }
    return this._selectors.relationSelector;
  }

  set notRelationSelector( not_relation_selector ) {
    not_relation_selector.not = true;
    this._selectors.notRelationSelector = not_relation_selector;
  }

  get notRelationSelector() {
    if ( ! this._selectors.notRelationSelector ) {
      this.notRelationSelector = new CSSRelationSelector( this );
    }
    return this._selectors.notRelationSelector;
  }

  set contentSelector( content_selector ) {
    this._selectors.contentSelector = content_selector;
  }

  get contentSelector() {
    if ( ! this._selectors.contentSelector ) {
      this.contentSelector = new CSSContentSelector( this );
    }
    return this._selectors.contentSelector;
  }

  set notContentSelector( not_content_selector ) {
    not_content_selector.not = true;
    this._selectors.notContentSelector = not_content_selector;
  }

  get notContentSelector() {
    if ( ! this._selectors.notContentSelector ) {
      this.notContentSelector = new CSSContentSelector( this );
    }
    return this._selectors.notContentSelector;
  }

  clearCSS() {
    super.clearCSS();
  }

}

export {CSSAttributeSelector, CSSContentSelector, CSSInputSelector, CSSRelationSelector, CSSStateSelector, CSSTagSelector, CSSTypeSelector};
export default CSSElementSelector;
