
import { Mixin, mix } from "../../mixwith.js";
import { CSSConstructionMixin } from "../css_construction_mixin.js";

export let CSSSelectorConstructionMixin = Mixin((superclass) => class extends mix(superclass).with(CSSConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_css_constructor = undefined, ...args ) {
    super( parent_css_constructor, ...args );
    this._slottedEnclosesNot = true;
    this._hostEnclosesNot = true;
  }

  /*******************
  *  Static Methods  *
  *******************/

  static notSelector( css ) {
    if ( css.length ) {
      let not_css = ':not(' + css + ')';
      return not_css;
    }
    return css;
  }

  static hostSelector( css ) {
    if ( css.length ) {
      let host_css = ':host(' + css + ')';
      return host_css;
    }
    return css;
  }

  static hostNotSelector( css ) {
    if ( css.length ) {
      let not_css = this.hostSelector(this.notSelector(css));
      return not_css;
    }
    return css;
  }

  static notHostSelector( css ) {
    if ( css.length ) {
      let not_css = this.notSelector(this.hostSelector(css));
      return not_css;
    }
    return css;
  }

  static slottedSelector( css ) {
    if ( css.length ) {
      let slotted_css = '::slotted(' + css + ')';
      return slotted_css;
    }
    return css;
  }

  static slottedNotSelector( css ) {
    if ( css.length ) {
      let not_css = this.slottedSelector(this.notSelector(css));
      return not_css;
    }
    return css;
  }

  static notSlottedSelector( css ) {
    if ( css.length ) {
      let not_css = this.notSelector(this.slottedSelector(css));
      return not_css;
    }
    return css;
  }

  /******************
  *  CSS Selectors  *
  ******************/

  get css() {
    if ( this._css ) {

      if ( this.host ) {
        if ( this.not ) {
          if ( this.hostEnclosesNot ) {
            this._css = this.constructor.hostNotSelector( this._css );
          }
          else {
            this._css = this.constructor.notHostSelector( this._css );
          }
        }
        else {
          this._css = this.constructor.hostSelector( this._css );
        }
      }
      else if ( this.slotted ) {
        if ( this.not ) {
          if ( this.slottedEnclosesNot ) {
            this._css = this.constructor.slottedNotSelector( this._css );
          }
          else {
            this._css = this.constructor.notSlottedSelector( this._css );
          }
        }
        else {
          this._css = this.constructor.slottedSelector( this._css );
        }
      }
      else if ( this.not ) {
        this._css = this.constructor.notSelector( this._css );
      }

    }
    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  set host( true_or_false ) {
    return this._host = true_or_false;
  }

  get host() {
    return this._host;
  }

  set slotted( true_or_false ) {
    return this._slotted = true_or_false;
  }

  get slotted() {
    return this._slotted;
  }

  set not( true_or_false ) {
    return this._not = true_or_false;
  }

  get not() {
    return this._not;
  }

  set slottedEnclosesNot( true_or_false ) {
    return this._slottedEnclosesNot = true_or_false;
  }

  set notEnclosesSlotted( true_or_false ) {
    return this.slottedEnclosesNot = ! true_or_false;
  }

  get slottedEnclosesNot() {
    return this._slottedEnclosesNot;
  }

  get notEnclosesSlotted() {
    return ! this.slottedEnclosesNot;
  }

  set hostEnclosesNot( true_or_false ) {
    return this._hostEnclosesNot = true_or_false;
  }

  set notEncloseshost( true_or_false ) {
    return this.hostEnclosesNot = ! true_or_false;
  }

  get hostEnclosesNot() {
    return this._hostEnclosesNot;
  }

  get notEnclosesHost() {
    return ! this.hostEnclosesNot;
  }

  set slottedNot( true_or_false ) {
    this.slotted = true_or_false;
    this.not = true_or_false;
    this.slottedEnclosesNot = true_or_false;
  }

  get slottedNot() {
    return this.slotted && this.not && this.slottedEnclosesNot;
  }

  set notSlotted( true_or_false ) {
    this.slotted = true_or_false;
    this.not = true_or_false;
    this.slottedEnclosesNot = ! true_or_false;
  }

  get notSlotted() {
    return this.slotted && this.not && ! this.slottedEnclosesNot;
  }

  set hostNot( true_or_false ) {
    this.host = true_or_false;
    this.not = true_or_false;
    this.hostEnclosesNot = true_or_false;
  }

  get hostNot() {
    return this.host && this.not && this.hostEnclosesNot;
  }

  set notHost( true_or_false ) {
    this.host = true_or_false;
    this.not = true_or_false;
    this.host = true_or_false;
    this.hostEnclosesNot = ! true_or_false;
  }

  get notHost() {
    return this._notHost;
  }

});
