
import { Mixin, mix } from "../../../mixwith.js";
import {CSSSelectorConstructionMixin} from "../css_selector_construction_mixin.js";

/*
    A single selector for tree relations.
*/
export class CSSContentSelector extends mix(Object).with(CSSSelectorConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_selector = undefined ) {
    super( parent_selector );
  }

  /********************
  *  Selector Output  *
  ********************/

  get css() {
    if ( ! this._css ) {

      if ( this.firstLetter ) {
        this._css = '::first-letter';
      }
      else if ( this.firstLine ) {
        this._css = '::first-line';
      }
      else if ( this.selection ) {
        this._css = '::selection';
      }
      else if ( this.before ) {
        this._css = '::before';
      }
      else if ( this.after ) {
        this._css = '::after';
      }

    }
    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  /*
      ::first-letter	p::first-letter	Selects the first letter of every <p> element
  */
  set firstLetter( true_or_false ) {
    this.clearCSS();
    return this._firstLetter = true_or_false;
  }

  get firstLetter() {
    return this._firstLetter;
  }

  /*
      ::first-line	p::first-line	Selects the first line of every <p> element
  */
  set firstLine( true_or_false ) {
    this.clearCSS();
    return this._firstLine = true_or_false;
  }

  get firstLine() {
    return this._firstLine;
  }

  /*
      ::selection	::selection	Selects the portion of an element that is selected by a user
  */
  set selection( true_or_false ) {
    this.clearCSS();
    return this._selection = true_or_false;
  }

  get selection() {
    return this._selection;
  }

  /*
      ::before	p::before	Insert something before the content of each <p> element
  */
  set before( true_or_false ) {
    this.clearCSS();
    return this._before = true_or_false;
  }

  get before() {
    return this._before;
  }

  /*
      ::after	p::after	Insert something after the content of each <p> element
  */
  set after( true_or_false ) {
    this.clearCSS();
    return this._after = true_or_false;
  }

  get after() {
    return this._after;
  }

  clearCSS() {
    super.clearCSS();
  }

}

export default CSSContentSelector;
