
import { Mixin, mix } from "../../../mixwith.js";
import {CSSSelectorConstructionMixin} from "../css_selector_construction_mixin.js";

/*
    A single selector.
*/
export class CSSInputSelector extends mix(Object).with(CSSSelectorConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_selector = undefined ) {
    super( parent_selector );
  }

  /********************
  *  Selector Output  *
  ********************/

  get css() {
    if ( ! this._css ) {

      this._css = '';

      if ( this.default ) {
        this._css += ':default';
      }

      if ( this.checked ) {
        this._css += ':checked';
      }

      if ( this.enabled ) {
        this._css += ':enabled';
      }
      else if ( this.disabled ) {
        this._css += ':disabled';
      }

      if ( this.focus ) {
        this._css += ':focus';
      }

      if ( this.inRange ) {
        this._css += ':in-range';
      }
      else if ( this.outOfRange ) {
        this._css += ':out-of-range';
      }

      if ( this.indeterminate ) {
        this._css += ':indeterminate';
      }

      if ( this.valid ) {
        this._css += ':valid';
      }
      else if ( this.invalid ) {
        this._css += ':invalid';
      }

      if ( this.readOnly ) {
        this._css += ':read-only';
      }
      else if ( this.readWrite ) {
        this._css += ':read-write';
      }

      if ( this.required ) {
        this._css += ':required';
      }
      else if ( this.optional ) {
        this._css += ':optional';
      }

      if ( this.placeholder ) {
        this._css += '::placeholder';
      }

    }
    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  /*
      :default	input:default	Selects the default <input> element
  */
  set default( true_or_false ) {
    this.clearCSS();
    return this._default = true_or_false;
  }

  get default() {
    return this._default;
  }

  /*
      :checked	input:checked	Selects every checked <input> element
  */
  set checked( true_or_false ) {
    this.clearCSS();
    return this._checked = true_or_false;
  }

  get checked() {
    return this._checked;
  }

  /*
      :enabled	input:enabled	Selects every enabled <input> element
  */
  set enabled( true_or_false ) {
    this.clearCSS();
    this._disabled = ! true_or_false;
    return this._enabled = true_or_false;
  }

  get enabled() {
    return this._enabled;
  }

  /*
      :disabled	input:disabled	Selects every disabled <input> element
  */
  set disabled( true_or_false ) {
    this.clearCSS();
    this._enabled = ! true_or_false;
    return this._disabled = true_or_false;
  }

  get disabled() {
    return this._disabled;
  }

  /*
      :focus	input:focus	Selects the input element which has focus
  */
  set focus( true_or_false ) {
    this.clearCSS();
    return this._focus = true_or_false;
  }

  get focus() {
    return this._focus;
  }

  /*
      :in-range	input:in-range	Selects input elements with a value within a specified range
  */
  set inRange( true_or_false ) {
    this.clearCSS();
    this._outOfRange = ! true_or_false;
    return this._inRange = true_or_false;
  }

  get inRange() {
    return this._inRange;
  }

  /*
      :out-of-range	input:out-of-range	Selects input elements with a value outside a specified range
  */
  set outOfRange( true_or_false ) {
    this.clearCSS();
    this._inRange = ! true_or_false;
    return this._outOfRange = true_or_false;
  }

  get outOfRange() {
    return this._outOfRange;
  }

  /*
      :indeterminate	input:indeterminate	Selects input elements that are in an indeterminate state
  */
  set indeterminate( true_or_false ) {
    this.clearCSS();
    return this._indeterminate = true_or_false;
  }

  get indeterminate() {
    return this._indeterminate;
  }

  /*
      :invalid	input:invalid	Selects all input elements with an invalid value
  */
  set invalid( true_or_false ) {
    this.clearCSS();
    return this._invalid = true_or_false;
  }

  get invalid() {
    return this._invalid;
  }

  /*
      :valid	input:valid	Selects all input elements with a valid value
  */
  set valid( true_or_false ) {
    this.clearCSS();
    this._indeterminate = ! true_or_false;
    return this._valid = true_or_false;
  }

  get valid() {
    return this._valid;
  }

  /*
      :read-only	input:read-only	Selects input elements with the "readonly" attribute specified
  */
  set readOnly( true_or_false ) {
    this.clearCSS();
    return this._readOnly = true_or_false;
  }

  get readOnly() {
    return this._readOnly;
  }

  /*
      :read-write	input:read-write	Selects input elements with the "readonly" attribute NOT specified
  */
  set readWrite( true_or_false ) {
    this.clearCSS();
    return this._readWrite = true_or_false;
  }

  get readWrite() {
    return this._readWrite;
  }

  /*
      :required	input:required	Selects input elements with the "required" attribute specified
  */
  set required( true_or_false ) {
    this.clearCSS();
    return this._required = true_or_false;
  }

  get required() {
    return this._required;
  }

  /*
      :optional	input:optional	Selects input elements with no "required" attribute
  */
  set optional( true_or_false ) {
    this.clearCSS();
    return this._optional = true_or_false;
  }

  get optional() {
    return this._optional;
  }

  /*
      ::placeholder	input::placeholder	Selects input elements with placeholder text
  */
  set placeholder( true_or_false ) {
    this.clearCSS();
    return this._placeholder = true_or_false;
  }

  get placeholder() {
    return this._placeholder;
  }

  clearCSS() {
    super.clearCSS();
  }

}

export default CSSInputSelector;
