
import { Mixin, mix } from "../../../mixwith.js";
import {CSSSelectorConstructionMixin} from "../css_selector_construction_mixin.js";

/*
    A single selector for tree relations.
*/
export class CSSRelationSelector extends mix(Object).with(CSSSelectorConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_selector = undefined ) {
    super( parent_selector );
  }

  /********************
  *  Selector Output  *
  ********************/

  get css() {
    if ( ! this._css ) {

      if ( this.root ) {
        this._css = ':root';
      }
      else if ( this.empty ) {
        this._css = ':empty';
      }
      else if ( this.onlyChild                      ||
              ( this.firstChild && this.lastChild ) ||
              ( ( this.nthChild === this.nthLastChild ) &&
                ( this.nthChild === 1 ) ) ) {
        this._css = ':only-child';
      }
      else if ( this.firstChild ) {
        this._css = ':first-child';
      }
      else if ( this.lastChild ) {
        this._css = ':last-child';
      }
      else if ( this.nthChild ) {
        this._css = ':nth-child(' + this.nthChild + ')';
        if ( this.nthLastChild ) {
          // Special case: nth-child and nth-last-child are equal.
          this._css += ':nth-last-child(' + this.nthLastChild + ')';
        }
      }
      else if ( this.nthLastChild ) {
        this._css = ':nth-last-child(' + this.nthLastChild + ')';
      }
      else {
        this._css = '';
      }

    }
    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  /*
      :root	:root	Selects the document's root element
  */
  set root( true_or_false ) {
    this.clearCSS();
    return this._root = true_or_false;
  }

  get root() {
    return this._root;
  }

  /*
      :empty	p:empty	Selects every <p> element that has no children (including text nodes)
  */
  set empty( true_or_false ) {
    this.clearCSS();
    return this._empty = true_or_false;
  }

  get empty() {
    return this._empty;
  }

  /*
      :first-child	p:first-child	Selects every <p> element that is the first child of its parent
  */
  set firstChild( true_or_false ) {
    this.clearCSS();
    return this._firstChild = true_or_false;
  }

  get firstChild() {
    return this._firstChild;
  }

  /*
      :last-child	p:last-child	Selects every <p> element that is the last child of its parent
  */
  set lastChild( true_or_false ) {
    this.clearCSS();
    return this._lastChild = true_or_false;
  }

  get lastChild() {
    return this._lastChild;
  }

  /*
      :only-child	p:only-child	Selects every <p> element that is the only child of its parent
  */
  set onlyChild( true_or_false ) {
    this.clearCSS();
    return this._onlyChild = true_or_false;
  }

  get onlyChild() {
    return this._onlyChild;
  }

  /*
      :nth-child(n)	p:nth-child(2)	Selects every <p> element that is the second child of its parent
  */
  set nthChild( n ) {
    this.clearCSS();
    return this._nthChild = n;
  }

  get nthChild() {
    return this._nthChild;
  }

  /*
      :nth-last-child(n)	p:nth-last-child(2)	Selects every <p> element that is the second child of its parent, counting from the last child
  */
  set nthLastChild( n ) {
    this.clearCSS();
    return this._nthLastChild = n;
  }

  get nthLastChild() {
    return this._nthLastChild;
  }

  clearCSS() {
    super.clearCSS();
  }

}

export default CSSRelationSelector;
