
import { Mixin, mix } from "../../../mixwith.js";
import {CSSSelectorConstructionMixin} from "../css_selector_construction_mixin.js";

/*
    A single selector.
*/
export class CSSStateSelector extends mix(Object).with(CSSSelectorConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_selector = undefined ) {
    super( parent_selector );
  }

  /********************
  *  Selector Output  *
  ********************/

  get css() {
    if ( ! this._css ) {

      this._css = '';

      if ( this.target ) {
        this._css += ':target';
      }

      if ( this.unvisited ) {
        this._css += ':link';
      }
      else if ( this.visited ) {
        this._css += ':visited';
      }

      if ( this.active ) {
        this._css += ':active';
      }
      else if ( this.hover ) {
        this._css += ':hover';
      }

    }
    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  /*
      :active	a:active	Selects the active link
  */
  set active( true_or_false ) {
    this.clearCSS();
    return this._active = true_or_false;
  }

  get active() {
    return this._active;
  }

  /*
      :hover	a:hover	Selects links on mouse over
  */
  set hover( true_or_false ) {
    this.clearCSS();
    return this._hover = true_or_false;
  }

  get hover() {
    return this._hover;
  }

  /*
      :link	a:link	Selects all unvisited links
  */
  set unvisited( true_or_false ) {
    this.clearCSS();
    return this._unvisited = true_or_false;
  }

  get unvisited() {
    return this._unvisited;
  }

  set link( true_or_false ) {
    return this.unvisited = true_or_false;
  }

  get link() {
    return this.unvisited;
  }

  /*
      :visited	a:visited	Selects all visited links
  */
  set visited( true_or_false ) {
    this.clearCSS();
    return this._visited = true_or_false;
  }

  get visited() {
    return this._visited;
  }

  /*
      :target	#news:target	Selects the current active #news element (clicked on a URL containing that anchor name)
  */
  set target( true_or_false ) {
    this.clearCSS();
    return this._target = true_or_false;
  }

  get target() {
    return this._target;
  }

  clearCSS() {
    super.clearCSS();
  }

}

export default CSSStateSelector;
