
import { Mixin, mix } from "../../../mixwith.js";
import {CSSSelectorConstructionMixin} from "../css_selector_construction_mixin.js";

/*
    A single selector.
*/
export class CSSTagSelector extends mix(Object).with(CSSSelectorConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_selector = undefined ) {
    super( parent_selector );
  }

  /********************
  *  Selector Output  *
  ********************/

  get css() {
    if ( ! this._css ) {
      this._css = '';
      if ( this.tag  ) {
        this._css += this.tag;
      }
      if ( this._classList  ) {
        this._css += this.classSelector;
      }
      if ( this._id ) {
        this._css += this.idSelector;
      }
    }
    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  /*
      element	p	Selects all <p> elements
      *	*	Selects all elements
  */
  set tag( tag_name ) {
    this.clearCSS();
    return this._tag = tag_name;
  }

  get tag() {
    return this._tag;
  }

  /*
      .class	.intro	Selects all elements with class="intro"
  */
  set classList( class_list_array ) {
    this.clearCSS();
    return this._classList = class_list_array;
  }

  get classList() {
    if ( ! this._classList ) {
      this._classList = [];
    }
    return this._classList;
  }

  addClass( ...class_names ) {
    this.clearCSS();
    this.classList.push( ...class_names );
  }

  get classSelector() {
    if ( ! this._classSelector ) {
      this._classSelector = this.classList.length ? '.' + this.classList.join('.') : '';
    }
    return this._classSelector;
  }

  /*
      #id	#firstname	Selects the element with id="firstname"
  */
  set id( identifier ) {
    this.clearCSS();
    return this._id = identifier;
  }

  get id() {
    return this._id;
  }

  get idSelector() {
    if ( ! this._idSelector ) {
      this._idSelector = this.id ? '#' + this.id : '';
    }
    return this._idSelector;
  }

  clearCSS() {
    super.clearCSS();
    delete this._classSelector;
  }

}

export default CSSTagSelector;
