
import { Mixin, mix } from "../../../mixwith.js";
import {CSSSelectorConstructionMixin} from "../css_selector_construction_mixin.js";

/*
    A single selector.
*/
export class CSSTypeSelector extends mix(Object).with(CSSSelectorConstructionMixin) {

  /*****************
  *  Constructors  *
  *****************/

  constructor( parent_selector = undefined ) {
    super( parent_selector );
  }

  /********************
  *  Selector Output  *
  ********************/

  get css() {
    if ( ! this._css ) {

      if ( this.onlyOfType                      ||
        ( this.firstOfType && this.lastOfType ) ||
        ( ( this.nthOfType === this.nthLastOfType ) &&
          ( this.nthOfType === 1 ) ) ) {
        this._css = ':only-of-type';
      }
      else if ( this.firstOfType ) {
        this._css = ':first-of-type';
      }
      else if ( this.lastOfType ) {
        this._css = ':last-of-type';
      }
      else if ( this.nthOfType ) {

        this._css = ':nth-of-type(' + this.nthOfType + ')';

        // Special case where nthOfType == nthLastOfType
        if ( this.nthLastOfType ) {
          this._css += ':nth-last-of-type(' + this.nthLastOfType + ')';
        }
      }
      else if ( this.nthLastOfType ) {
        this._css = ':nth-last-of-type(' + this.nthLastOfType + ')';
      }
      else {
        this._css = '';
      }

      if ( this.language ) {
        this._css += ':lang(' + this.language + ')';
      }

    }
    return this._css;
  }

  /***************
  *  Properties  *
  ***************/

  /*
      :only-of-type	p:only-of-type	Selects every <p> element that is the only <p> element of its parent
  */
  set onlyOfType( true_or_false ) {
    this.clearCSS();
    return this._onlyOfType = true_or_false;
  }

  get onlyOfType() {
    return this._onlyOfType;
  }

  /*
      :first-of-type	p:first-of-type	Selects every <p> element that is the first <p> element of its parent
  */
  set firstOfType( true_or_false ) {
    this.clearCSS();
    return this._firstOfType = true_or_false;
  }

  get firstOfType() {
    return this._firstOfType;
  }

  /*
      :last-of-type	p:last-of-type	Selects every <p> element that is the last <p> element of its parent
  */
  set lastOfType( true_or_false ) {
    this.clearCSS();
    return this._lastOfType = true_or_false;
  }

  get lastOfType() {
    return this._lastOfType;
  }

  /*
      :nth-of-type(n)	p:nth-of-type(2)	Selects every <p> element that is the second <p> element of its parent
  */
  set nthOfType( n ) {
    this.clearCSS();
    return this._nthOfType = n;
  }

  get nthOfType() {
    return this._nthOfType;
  }

  /*
      :nth-last-of-type(n)	p:nth-last-of-type(2)	Selects every <p> element that is the second <p> element of its parent, counting from the last child
  */
  set nthLastOfType( n ) {
    this.clearCSS();
    return this._nthLastOfType = n;
  }

  get nthLastOfType() {
    return this._nthLastOfType;
  }

  /*
      :lang(language)	p:lang(it)	Selects every <p> element with a lang attribute equal to "it" (Italian)
  */
  set language( language_id ) {
    this.clearCSS();
    return this._language = language_id;
  }

  get language() {
    return this._language;
  }

  set lang( language_id ) {
    return this.language = language_id;
  }

  get lang() {
    return this.language;
  }

  clearCSS() {
    super.clearCSS();
  }

}

export default CSSTypeSelector;
