import { CSSSelector } from '../../css_selector.js';

fixture `CSSSelector - Attributes`;

test( 'attributeSelectorNameShouldSelectAttributeName', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.shouldSelectAttributeName = true;
  await test.expect(css_selector.css).eql(
    '[some-attribute]'
  );
});

test( 'attributeSelectorNameShouldNotSelectAttributeName', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  await test.expect(css_selector.css).eql(
    ''
  );
});

test( 'attributeSelectorNameIsValue', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.isValue = 'some-value';
  await test.expect(css_selector.css).eql(
    '[some-attribute="some-value"]'
  );
});

test( 'attributeSelectorNameBeginsWithValue', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.beginsWithValue = 'some-val';
  await test.expect(css_selector.css).eql(
    '[some-attribute^="some-val"]'
  );
});

test( 'attributeSelectorNameEndsWithValue', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.endsWithValue = 'me-value';
  await test.expect(css_selector.css).eql(
    '[some-attribute$="me-value"]'
  );
});

test( 'attributeSelectorNameBeginsWithValueAndEndsWithValue', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.beginsWithValue = 'some';
  attribute_selector.endsWithValue = 'value';
  await test.expect(css_selector.css).eql(
    '[some-attribute^="some"][some-attribute$="value"]'
  );
});

test( 'attributeSelectorNameContainsValues', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.addContainsValues('some', 'value');
  await test.expect(css_selector.css).eql(
    '[some-attribute*="some"][some-attribute*="value"]'
  );
});

test( 'attributeSelectorNameBeginsWithValueAndContainsValuesAndEndsWithValue', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.beginsWithValue = 'some';
  attribute_selector.addContainsValues('-');
  attribute_selector.endsWithValue = 'value';
  await test.expect(css_selector.css).eql(
    '[some-attribute^="some"][some-attribute*="-"][some-attribute$="value"]'
  );
});

test( 'attributeSelectorNameContainsSpacedWords', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.addContainsSpacedWords('some', 'value');
  await test.expect(css_selector.css).eql(
    '[some-attribute~="some"][some-attribute~="value"]'
  );
});

test( 'attributeSelectorNameBeginsWithValueAndContainsSpacedWordsAndEndsWithValue', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.beginsWithValue = 'some';
  attribute_selector.addContainsSpacedWords('spaced', 'words');
  attribute_selector.endsWithValue = 'value';
  // matches "some spaced words value"
  await test.expect(css_selector.css).eql(
    '[some-attribute^="some"][some-attribute~="spaced"][some-attribute~="words"][some-attribute$="value"]'
  );
});

test( 'attributeSelectorNameBeginsWithDashedWord', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.beginsWithDashedWord = 'some';
  await test.expect(css_selector.css).eql(
    '[some-attribute|="some"]'
  );
});

test( 'attributeSelectorNameBeginsWithDashedWordAndEndsWithValue', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.beginsWithDashedWord = 'some';
  attribute_selector.endsWithValue = 'value';
  await test.expect(css_selector.css).eql(
    '[some-attribute|="some"][some-attribute$="value"]'
  );
});

test( 'attributeSelectorNameBeginsWithDashedWordAndContainsValuesAndEndsWithValue', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.beginsWithDashedWord = 'some';
  attribute_selector.addContainsValues('arb', 'tex');
  attribute_selector.endsWithValue = 'value';
  // matches "some-arbitrary text value"
  await test.expect(css_selector.css).eql(
    '[some-attribute|="some"][some-attribute*="arb"][some-attribute*="tex"][some-attribute$="value"]'
  );
});

test( 'attributeSelectorNameBeginsWithDashedWordAndContainsSpacedWordsAndEndsWithValue', async test => {
  let css_selector = new CSSSelector();
  let attribute_selector = css_selector.addAttributeSelector('some-attribute');
  attribute_selector.beginsWithDashedWord = 'some';
  attribute_selector.addContainsSpacedWords('spaced', 'words');
  attribute_selector.endsWithValue = 'value';
  // matches "some-spaced words value"
  await test.expect(css_selector.css).eql(
    '[some-attribute|="some"][some-attribute~="spaced"][some-attribute~="words"][some-attribute$="value"]'
  );
});
